@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.102';
+our $VERSION = '1.801';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
@@ -127,7 +127,7 @@ sub new {
    MCE::Util::_sock_pair($_Q, qw(_ar_sock _aw_sock)) if $_Q->{_await};
 
    if (exists $_argv{queue} && scalar @{ $_argv{queue} }) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
 
    return $_Q;
@@ -156,7 +156,7 @@ sub clear {
    }
    else {
       if ($_Q->_has_data()) {
-         1 until sysread $_Q->{_qr_sock}, $_next, 1;
+         sysread $_Q->{_qr_sock}, $_next, 1;
       }
       %{ $_Q->{_datp} } = ();
       @{ $_Q->{_datq} } = ();
@@ -174,7 +174,7 @@ sub enqueue {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
 
    push @{ $_Q->{_datq} }, @_;
@@ -193,7 +193,7 @@ sub enqueuep {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
 
    $_Q->_enqueuep($_p, @_);
@@ -208,7 +208,7 @@ sub dequeue {
    my ($_Q, $_cnt) = @_;
    my (@_items, $_buf, $_next, $_pending);
 
-   1 until sysread $_Q->{_qr_sock}, $_next, 1;  # block
+   sysread $_Q->{_qr_sock}, $_next, 1;  # block
 
    if (defined $_cnt && $_cnt ne '1') {
       @_items = $_Q->_dequeue($_cnt);
@@ -223,7 +223,7 @@ sub dequeue {
          $_pending = int($_pending / $_cnt) if (defined $_cnt);
          if ($_pending) {
             $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
-            for (1 .. $_pending) { 1 until syswrite $_Q->{_qw_sock}, $LF }
+            for (1 .. $_pending) { syswrite $_Q->{_qw_sock}, $LF }
          }
          $_Q->{_dsem} = $_pending;
       }
@@ -233,7 +233,7 @@ sub dequeue {
    }
    else {
       # Otherwise, never to exceed one byte in the channel
-      if ($_Q->_has_data()) { 1 until syswrite $_Q->{_qw_sock}, $LF }
+      if ($_Q->_has_data()) { syswrite $_Q->{_qw_sock}, $LF }
    }
 
    $_Q->{_nb_flag} = 0;
@@ -286,7 +286,7 @@ sub insert {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
 
    if (abs($_i) > scalar @{ $_Q->{_datq} }) {
@@ -330,7 +330,7 @@ sub insertp {
    return unless (scalar @_);
 
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite $_Q->{_qw_sock}, $LF;
+      syswrite $_Q->{_qw_sock}, $LF;
    }
 
    if (exists $_Q->{_datp}->{$_p} && scalar @{ $_Q->{_datp}->{$_p} }) {
@@ -611,7 +611,7 @@ MCE::Shared::Queue - Hybrid-queue helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Queue version 1.102
+This document describes MCE::Shared::Queue version 1.801
 
 =head1 SYNOPSIS
 
